@@ -14,11 +14,11 @@ using System.Collections;
 using System.Runtime.InteropServices;
 
 namespace Exif {
-	public enum ExifTag {
+	public enum Tag {
 		InteroperabilityIndex		= 0x0001,
 		InteroperabilityVersion	        = 0x0002,
 		ImageWidth 			= 0x0100,
-		ImageLength 			= 0x0101,
+		ImageHeight 			= 0x0101,
 		BitsPersample 	         	= 0x0102,
 		Compression 			= 0x0103,
 		PhotometricInterpretation 	= 0x0106,
@@ -52,7 +52,7 @@ namespace Exif {
 		ReferenceBlackWhite		= 0x0214,
 		RelatedImageFileFormat   	= 0x1000,
 		RelatedImageWidth		= 0x1001,
-		RelatedImageLength		= 0x1002,
+		RelatedImageHeight		= 0x1002,
 		CFARepeatPatternDim		= 0x828d,
 		CFAPattern			= 0x828e,
 		BatteryLevel			= 0x828f,
@@ -129,7 +129,7 @@ namespace Exif {
 		PimIfdPointer              = 0xc4a5
 	}
 	
-	public enum ExifByteOrder {
+	public enum ByteOrder {
 		Motorola,
 		Intel
 	}
@@ -145,7 +145,7 @@ namespace Exif {
 		SRational = 10
 	}
 	
-	public enum ExifIfd {
+	public enum Ifd {
 		Zero = 0,
 		One,
 		Exif,
@@ -157,16 +157,16 @@ namespace Exif {
 	internal class ExifUtil {
 		
 		[DllImport ("libexif.dll")]
-		static extern IntPtr exif_tag_get_name (ExifTag tag);
+		static extern IntPtr exif_tag_get_name (Tag tag);
 		
 		[DllImport ("libexif.dll")]
-		static extern IntPtr exif_tag_get_title (ExifTag tag);
+		static extern IntPtr exif_tag_get_title (Tag tag);
 		
 		[DllImport ("libexif.dll")]
-		static extern IntPtr exif_tag_get_description (ExifTag tag);
+		static extern IntPtr exif_tag_get_description (Tag tag);
 		
 		[DllImport ("libexif.dll")]
-		static extern IntPtr exif_byte_order_get_name (ExifByteOrder order);
+		static extern IntPtr exif_byte_order_get_name (ByteOrder order);
 		
 		[DllImport ("libexif.dll")]
 		static extern IntPtr exif_format_get_name (ExifFormat format);
@@ -175,28 +175,28 @@ namespace Exif {
 		static extern char exif_format_get_size (ExifFormat format);
 		
 		[DllImport ("libexif.dll")]
-		static extern IntPtr exif_ifd_get_name (ExifIfd ifd);
+		static extern IntPtr exif_ifd_get_name (Ifd ifd);
 		
-		public static string GetTagName (ExifTag tag)
+		public static string GetTagName (Tag tag)
 		{
 			
 			IntPtr raw_ret = exif_tag_get_name (tag);
 			return Marshal.PtrToStringAnsi (raw_ret);
 		}
 		
-		public static string GetTagTitle (ExifTag tag)
+		public static string GetTagTitle (Tag tag)
 		{
 			IntPtr raw_ret = exif_tag_get_title (tag);
 			return Marshal.PtrToStringAnsi (raw_ret);
 		}
 		
-		public static string GetTagDescription (ExifTag tag)
+		public static string GetTagDescription (Tag tag)
 		{
 			IntPtr raw_ret = exif_tag_get_description (tag);
 			return Marshal.PtrToStringAnsi (raw_ret);
 		}
 		
-		public static string GetByteOrderName (ExifByteOrder order)
+		public static string GetByteOrderName (ByteOrder order)
 		{
 			IntPtr raw_ret = exif_byte_order_get_name (order);
 			return Marshal.PtrToStringAnsi (raw_ret);
@@ -213,24 +213,24 @@ namespace Exif {
 			return exif_format_get_size (format);
 		}
 		
-		public static string GetIfdName (ExifIfd ifd)
+		public static string GetIfdName (Ifd ifd)
 		{			
 			IntPtr raw_ret = exif_ifd_get_name (ifd);
 			return Marshal.PtrToStringAnsi (raw_ret);
 		}
 		
-		public static string GetIfdNameExtended (ExifIfd ifd)
+		public static string GetIfdNameExtended (Ifd ifd)
 		{
 			switch (ifd) {
-			case ExifIfd.Zero:
+			case Ifd.Zero:
 				return Mono.Posix.Catalog.GetString ("Image Directory");
-			case ExifIfd.One:
+			case Ifd.One:
 				return Mono.Posix.Catalog.GetString ("Thumbnail Directory");
-			case ExifIfd.Exif:
+			case Ifd.Exif:
 				return Mono.Posix.Catalog.GetString ("Exif Directory");
-			case ExifIfd.Gps:
+			case Ifd.Gps:
 				return Mono.Posix.Catalog.GetString ("GPS Directory");
-			case ExifIfd.InterOperability:
+			case Ifd.InterOperability:
 				return Mono.Posix.Catalog.GetString ("InterOperability Directory");
 			default:
 				return Mono.Posix.Catalog.GetString ("Unknown Directory");
@@ -284,11 +284,11 @@ namespace Exif {
 	
 	[StructLayout(LayoutKind.Sequential)]
 	internal unsafe struct _ExifContent {
-		IntPtr entries;
-		uint count; 
-		IntPtr parent;
+		public IntPtr entries;
+		public uint count; 
+		public IntPtr parent;
 		
-		IntPtr priv;
+		public IntPtr priv;
 	}
 	
 	public class ExifContent : ExifObject {
@@ -324,7 +324,7 @@ namespace Exif {
 		[DllImport ("libexif.dll")]
 		internal static extern void exif_content_add_entry (HandleRef content, HandleRef entry);
 		
-		public ExifEntry Lookup (ExifTag tag)
+		public ExifEntry Lookup (Tag tag)
 		{
 			Assemble ();
 			
@@ -344,7 +344,7 @@ namespace Exif {
 			return entries.Contains (entry);
 		}
 
-		public ExifEntry GetEntry (ExifTag tag)
+		public ExifEntry GetEntry (Tag tag)
 		{
 			Assemble ();
 			
@@ -412,7 +412,7 @@ namespace Exif {
 	
 	[StructLayout(LayoutKind.Sequential)]
 	internal struct _ExifEntry {
-		public ExifTag tag;
+		public Tag tag;
 		public int format;
 		public uint components;
 		public IntPtr data;
@@ -420,7 +420,7 @@ namespace Exif {
 		
 		public IntPtr parent;
 		
-		IntPtr priv;
+		public IntPtr priv;
 	}
 	
 	
@@ -461,16 +461,16 @@ namespace Exif {
 		internal static extern IntPtr exif_entry_new ();
 
 		[DllImport ("libexif.dll")]
-		internal static extern void exif_entry_initialize (HandleRef handle, ExifTag tag);
+		internal static extern void exif_entry_initialize (HandleRef handle, Tag tag);
 
-		public ExifEntry (ExifContent parent, ExifTag tag)
+		public ExifEntry (ExifContent parent, Tag tag)
 		{
 			handle = new HandleRef (this, exif_entry_new ());
 			parent.Add (this);
 			this.Reset (tag);
 		}
 		
-		public void Reset (ExifTag tag)
+		public void Reset (Tag tag)
 		{
 			unsafe {
 				// Free any exsting data so that _initialize will actually set the data
@@ -482,9 +482,9 @@ namespace Exif {
 			exif_entry_initialize (handle, tag);
 
 			//FIXME the month string in time fields in libexif ix currently broken so we do our own. 
-			if (tag == ExifTag.DateTime
-			    || tag == ExifTag.DateTimeOriginal
-			    || tag == ExifTag.DateTimeDigitized)
+			if (tag == Tag.DateTime
+			    || tag == Tag.DateTimeOriginal
+			    || tag == Tag.DateTimeDigitized)
 				this.SetData (System.DateTime.Now);
 
 		}
@@ -506,7 +506,7 @@ namespace Exif {
 			}
 		}
 		
-		public ExifTag Tag {
+		public Tag Tag {
 			get {
 				unsafe {
 					return _handle->tag;
@@ -532,7 +532,7 @@ namespace Exif {
 			}
 		}
 		
-		public void SetData (byte [] data, bool check_type)
+		public void SetData (byte [] data, int size)
 		{
 			unsafe {
 				if (data == null || data.Length == 0)
@@ -547,18 +547,27 @@ namespace Exif {
 				_handle->size = (uint) data.Length;
 				// This needs to be set per type as well but
 				// we do it here as well
-				_handle->components = (uint) data.Length;
+				_handle->components = (uint) (data.Length / size);
 			}
 		}
 		
 		public void SetData (byte []data)
 		{
-			SetData (data, true);
+			SetData (data, data.Length);
 		}
 
+		public void SetData (uint s)
+		{
+			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel));
+		}
+
+		public void SetData (ushort s)
+		{
+			this.SetData (FSpot.BitConverter.GetBytes (s, this.ByteOrder == ByteOrder.Intel));
+		}	    
+
 		public void SetData (ushort [] data)
 		{
-
 		} 
 		
 		public void SetData (short [] data)
@@ -573,7 +582,7 @@ namespace Exif {
 			System.Text.Encoding.UTF8.GetBytes (value, 0, value.Length, tmp, 0);
 			tmp[len] = 0;
 			System.Console.WriteLine ("value = {0} len = {1}", value, len);
-			SetData (tmp, false);
+			SetData (tmp, 1);
 		}
 
 		public void SetData (System.DateTime time)
@@ -584,7 +593,7 @@ namespace Exif {
 		private unsafe void PutBytes (byte *dest, byte *src, int count)
 		{
 			int i = 0;
-			if (System.BitConverter.IsLittleEndian == (this.ByteOrder == ExifByteOrder.Intel)) {
+			if (System.BitConverter.IsLittleEndian == (this.ByteOrder == ByteOrder.Intel)) {
 				for (i = 0; i < count; i++) {
 					//System.Console.WriteLine ("Copying normal byte [{0}]= {1}", i, src[i]);
 					dest [i] = src [i];
@@ -646,7 +655,7 @@ namespace Exif {
 			return null;
 		}
 
-		public ExifByteOrder ByteOrder
+		public ByteOrder ByteOrder
 		{
 			get {
 				return parent.Parent.GetByteOrder ();
@@ -656,7 +665,7 @@ namespace Exif {
 		public string Description 
 		{
 			get {
-					return ExifUtil.GetTagDescription (Tag);
+				return ExifUtil.GetTagDescription (Tag);
 			}
 		}
 		
@@ -727,16 +736,16 @@ namespace Exif {
 	
 	[StructLayout(LayoutKind.Sequential)]
 	internal struct _ExifData {
-		IntPtr ifd0;
-		IntPtr ifd1;
-		IntPtr ifd_exif;
-		IntPtr ifd_gps;
-		IntPtr ifd_interop;
+		internal IntPtr ifd0;
+		internal IntPtr ifd1;
+		internal IntPtr ifd_exif;
+		internal IntPtr ifd_gps;
+		internal IntPtr ifd_interop;
 
 		internal IntPtr  data;
 		internal int     size;
 		
-		IntPtr priv;
+		internal IntPtr priv;
 	}
 	
 	public class ExifData : ExifObject {
@@ -760,7 +769,12 @@ namespace Exif {
 		
 		[DllImport ("libexif.dll")]
 		internal static extern IntPtr exif_data_new_from_data (byte [] data, uint size);
-		
+
+		public ExifData (byte [] data)
+		{
+			handle = new HandleRef (this, exif_data_new_from_data (data, (uint) data.Length));
+		}
+
 		public ExifData (byte [] data, uint size)
 		{
 			handle = new HandleRef (this, exif_data_new_from_data (data, size));
@@ -773,8 +787,7 @@ namespace Exif {
 		internal static extern IntPtr malloc (uint size);
 		
 		[DllImport ("libexif.dll")]
-		private static extern void exif_data_save_data (HandleRef handle, out IntPtr content, out uint size);
-		
+		private static extern void exif_data_save_data (HandleRef handle, out IntPtr content, out uint size);		
 		public byte [] Save ()
 		{
 			Byte [] content = null;
@@ -803,7 +816,15 @@ namespace Exif {
 			exif_data_unref (handle);
 		}
 		
-		public ExifContent GetContents (ExifIfd ifd)
+		[DllImport ("libexif.dll")]
+		internal static extern void exif_data_dump (HandleRef data);
+
+		public void Dump ()
+		{
+			exif_data_dump (handle);
+		}
+		
+		public ExifContent GetContents (Ifd ifd)
 		{
 			Assemble ();
 
@@ -818,9 +839,9 @@ namespace Exif {
 		}
 
 		[DllImport("libexif.dll")]
-		internal static extern ExifByteOrder exif_data_get_byte_order (HandleRef handle);
+		internal static extern ByteOrder exif_data_get_byte_order (HandleRef handle);
 		
-		public ExifByteOrder GetByteOrder ()
+		public ByteOrder GetByteOrder ()
 		{
 			return exif_data_get_byte_order (handle);
 		}
@@ -835,7 +856,7 @@ namespace Exif {
 			ifds.Add (new ExifContent (this, content));
 		}
 		
-		public ExifEntry LookupFirst (ExifTag tag)
+		public ExifEntry LookupFirst (Tag tag)
 		{
 			Assemble ();
 			foreach (ExifContent content in ifds) {
@@ -849,7 +870,7 @@ namespace Exif {
 			return null;
 		}
 
-		public string LookupFirstValue (ExifTag tag)
+		public string LookupFirstValue (Tag tag)
 		{
 			ExifEntry entry = LookupFirst (tag);
 			if (entry != null) {
