@@ -7,6 +7,7 @@ using System;
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
 	bool recurse;
+	bool copy;
 	string [] base_paths;
 	Tag [] tags;
 
@@ -17,13 +18,19 @@ public class FileImportBackend : ImportBackend {
 
 	private void AddPath (string path)
 	{
-		if (path.ToLower().EndsWith (".jpg") || path.ToLower().EndsWith (".jpeg"))
+		if (FSpot.ImageFile.HasLoader (path))
 			file_paths.Add (path);
 	}
 
 	private void GetListing (System.IO.DirectoryInfo info)
 	{
-		GetListing (info, info.GetFiles (), recurse);
+		try {
+			GetListing (info, info.GetFiles (), recurse);
+		} catch (System.UnauthorizedAccessException e) {
+			System.Console.WriteLine ("Unable to access directory {0}", info.FullName);
+		} catch (System.Exception e) {
+			System.Console.WriteLine ("{0}", e.ToString ());
+		}
 	}
 
 	private void GetListing (System.IO.DirectoryInfo info, System.IO.FileInfo [] files, bool recurse)
@@ -81,6 +88,44 @@ public class FileImportBackend : ImportBackend {
 		return file_paths.Count;
 	}
 
+	public static string UniqueName (string path, string filename)
+	{
+		int i = 1;
+		string dest = System.IO.Path.Combine (path, filename);
+
+		while (System.IO.File.Exists (dest)) {
+			string numbered_name = String.Format ("{0}-{1}{2}", 
+							      System.IO.Path.GetFileNameWithoutExtension (filename),
+							      i++,
+							      System.IO.Path.GetExtension (filename));
+			
+			dest = System.IO.Path.Combine (path, numbered_name);
+		}
+		
+		return dest;
+	}
+
+	public static string ChooseLocation (string path)
+	{
+		string name = System.IO.Path.GetFileName (path);
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		DateTime time = img.Date ();
+		
+		string dest_dir = String.Format ("{0}{1}{2}{1}{3}{1}{4}",
+						 FSpot.Global.PhotoDirectory,
+						 System.IO.Path.DirectorySeparatorChar,
+						 time.Year,
+						 time.Month,
+						 time.Day);
+		
+		if (!System.IO.Directory.Exists (dest_dir))
+			System.IO.Directory.CreateDirectory (dest_dir);
+		
+		string dest = UniqueName (dest_dir, name);
+		
+		return dest;
+	}
+
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
 		if (file_paths == null)
@@ -90,8 +135,17 @@ public class FileImportBackend : ImportBackend {
 			throw new Exception ("Already finished");
 
 		// FIXME Need to get the EXIF info etc.
-		photo = store.Create (file_paths [this.count] as string, out thumbnail);
+		string path = (string) file_paths [this.count];
+
+		if (copy) {
+			string dest = ChooseLocation (path);
+
+			System.IO.File.Copy (path, dest);
+			path = dest;
+		}
 		
+		photo = store.Create (path, out thumbnail);
+
 		if (tags != null) {
 			foreach (Tag t in tags) {
 				photo.AddTag (t);
@@ -112,8 +166,17 @@ public class FileImportBackend : ImportBackend {
 		if (imported_photos == null)
 			throw new Exception ("Not doing anything");
 
-		foreach (Photo p in imported_photos)
+		foreach (Photo p in imported_photos) {
+			if (copy) {
+				try {
+					System.IO.File.Delete (p.DefaultVersionPath);
+				} catch (System.Exception e) {
+					System.Console.WriteLine (e);
+				}
+			}
+			
 			store.Remove (p);
+		}
 
 		Finish ();
 	}
@@ -133,11 +196,12 @@ public class FileImportBackend : ImportBackend {
 		count = 0;
 	}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse) : this (store, base_paths, recurse, null) {}
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse) : this (store, base_paths, false, recurse, null) {}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse, Tag [] tags)
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, Tag [] tags)
 	{
 		this.store = store;
+		this.copy = copy;
 		this.base_paths = base_paths;
 		this.recurse = recurse;
 		this.tags = tags;
@@ -157,7 +221,7 @@ public class FileImportBackend : ImportBackend {
 
 		Db db = new Db (path, true);
 
-		FileImportBackend import = new FileImportBackend (db.Photos, args [0], true);
+		FileImportBackend import = new FileImportBackend (db.Photos, args [0],true);
 
 		Console.WriteLine ("Preparing...");
 
