@@ -15,12 +15,17 @@ using System;
 using System.Reflection;
 using System.Collections;
 using System.IO;
+using FSpot;
 
 public class IconView : Gtk.Layout {
 
 	// Public properties.
 	FSpot.PixbufCache cache;
 
+	/* preserve the scroll postion when possible */
+	private bool scroll;
+	private double scroll_value;
+
 	/* Width of the thumbnails. */
 	protected int thumbnail_width = 128;
 	public int ThumbnailWidth {
@@ -74,7 +79,10 @@ public class IconView : Gtk.Layout {
 	private bool display_dates = true;
 	public bool DisplayDates {
 		get {
-			return display_dates;
+			if (cell_width > 100)
+				return display_dates;
+			else 
+				return false;
 		}
 
 		set {
@@ -84,7 +92,7 @@ public class IconView : Gtk.Layout {
 	}
 
 	// Size of the frame around the thumbnail.
-	protected const int CELL_BORDER_WIDTH = 10;
+	protected int cell_border_width = 10;
 
 	// Border around the scrolled area.
 	protected const int BORDER_SIZE = 6;
@@ -93,13 +101,13 @@ public class IconView : Gtk.Layout {
 	private const int SELECTION_THICKNESS = 5;
 
 	// Size of the tag icon in the view.
-	protected const int TAG_ICON_SIZE = 16;
+	protected int tag_icon_size = 16;
 
 	// Horizontal spacing between the tag icons
-	protected const int TAG_ICON_HSPACING = 2;
+	protected int tag_icon_hspacing = 2;
 
 	// Vertical spacing between the thumbnail and the row of tag icons.
-	protected const int TAG_ICON_VSPACING = 3;
+	protected int tag_icon_vspacing = 3;
 
 	// Various other layout values.
 	protected int cells_per_row;
@@ -112,9 +120,6 @@ public class IconView : Gtk.Layout {
 	private int y_offset;
 	private int x_offset;
 
-	// Hash of all the order number of the items that are selected.
-	private Hashtable selected_cells;
-
 	// Drag and drop bookkeeping. 
 	private int click_x, click_y;
 
@@ -144,18 +149,13 @@ public class IconView : Gtk.Layout {
 	public delegate void DoubleClickedHandler (IconView view, int clicked_item);
 	public event DoubleClickedHandler DoubleClicked;
 
-	public delegate void SelectionChangedHandler (IconView view);
-	public event SelectionChangedHandler SelectionChanged;
-
-
 	// Public API.
 	public IconView (IntPtr raw) : base (raw) {}
 
-	public IconView () : base (null, null)
+	protected IconView () : base (null, null)
 	{
 		cache = new FSpot.PixbufCache ();
 		cache.OnPixbufLoaded += HandlePixbufLoaded;
-		selected_cells = new Hashtable ();
 
 		ScrollAdjustmentsSet += new ScrollAdjustmentsSetHandler (HandleScrollAdjustmentsSet);
 		
@@ -163,7 +163,7 @@ public class IconView : Gtk.Layout {
 		ButtonReleaseEvent += new ButtonReleaseEventHandler (HandleButtonReleaseEvent);
 		KeyPressEvent += new KeyPressEventHandler (HandleKeyPressEvent);
 
-		DestroyEvent += new DestroyEventHandler (HandleDestroyEvent);
+		Destroyed += HandleDestroyed;
 
 		AddEvents ((int) EventMask.KeyPressMask
 			   | (int) EventMask.KeyReleaseMask 
@@ -177,8 +177,36 @@ public class IconView : Gtk.Layout {
 	public IconView (FSpot.IBrowsableCollection collection) : this () 
 	{
 		this.collection = collection;
+		this.selection = new SelectionCollection (collection);
+		
+		collection.Changed += HandleChanged;
+		collection.ItemChanged += HandleItemChanged;
+
+		selection.DetailedChanged += HandleSelectionChanged;
+	}
+
+	private void HandleSelectionChanged (FSpot.IBrowsableCollection collection, int [] ids)
+	{
+		if (ids == null)
+			QueueDraw ();
+		else 
+			foreach (int id in ids)
+				InvalidateCell (id);
+	}
+
+	private void HandleChanged (FSpot.IBrowsableCollection sender)
+	{
+		// FIXME we should probably try to merge the selection forward
+		// but it needs some thought to be efficient.
+		QueueResize ();
 	}
 	
+	private void HandleItemChanged (FSpot.IBrowsableCollection sender, int item)
+	{
+		UpdateThumbnail (item);
+		InvalidateCell (item);
+	}
+
 	//
 	// IPhotoSelection
 	//
@@ -190,34 +218,236 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	public int [] SelectedIdxs {
+	protected SelectionCollection selection;
+	public SelectionCollection Selection {
 		get {
-			int [] selection = new int [selected_cells.Count];
-
-			int i = 0;
-			foreach (int cell in selected_cells.Keys)
-				selection [i ++] = cell;
-
-			Array.Sort (selection);
 			return selection;
 		}
 	}
 
-	public int SelectedIdxCount 
-	{
-		get {
-			return selected_cells.Count;
+	// FIXME right now a selection change triggers a complete view redraw
+	// This should be optimized away by directly notifyiing the view of changed
+	// indexes rather than having the view connect to the collection.Changed event.      
+	public class SelectionCollection : IBrowsableCollection {
+		IBrowsableCollection parent;
+		Hashtable selected_cells;
+		int [] selection;
+		IBrowsableItem [] items;
+		IBrowsableItem [] old;
+
+		public SelectionCollection (IBrowsableCollection collection)
+		{
+			this.selected_cells = new Hashtable ();
+			this.parent = collection;
+			this.parent.Changed += HandleParentChanged;
+			this.parent.ItemChanged += HandleParentItemChanged;
+		}
+
+		private void HandleParentChanged (IBrowsableCollection collection)
+		{
+			IBrowsableItem [] local = old;
+			selected_cells.Clear ();
+			ClearCached ();
+
+			if (old != null) {
+				int i = 0;
+
+				for (i = 0; i < local.Length; i++) {
+					int parent_index = parent.IndexOf (local [i]);
+					if (parent_index >= 0)
+						this.Add (parent_index, false);
+				}
+			}
+
+			// Call the directly so that we don't reset old immediately this way the old selection
+			// set isn't actually lost until we change it.
+			if (this.Changed != null)
+				Changed (this);
+
+			if (this.DetailedChanged != null)
+				DetailedChanged (this, null);
+
+		}
+
+		private void HandleParentItemChanged (IBrowsableCollection collection, int parent_index)
+		{
+			int local_index = this.IndexOf (parent_index);
+			//System.Console.WriteLine ("parent = {0} local = {1}", parent_index, local_index);
+			if (local_index >= 0 && this.ItemChanged != null)
+				this.ItemChanged (collection, local_index);
+		}
+
+		public int [] Ids {
+			get {
+				if (selection != null)
+					return selection;
+				
+				selection = new int [selected_cells.Count];
+				
+				int i = 0;
+				foreach (int cell in selected_cells.Values)
+					selection [i ++] = cell;
+				
+				Array.Sort (selection);
+				return selection;
+			}
+		}
+		
+		public IBrowsableItem this [int index] {
+			get {
+				int [] ids = this.Ids;
+				return parent [ids[index]];
+			}
+		}
+
+		public IBrowsableItem [] Items {
+			get {
+				if (items != null)
+					return items;
+				
+				int [] ids = this.Ids;
+				items = new IBrowsableItem [ids.Length];
+				for (int i = 0; i < items.Length; i++) {
+					items [i] = parent [ids[i]];
+				}
+				return items;
+			}
+		}
+
+		public void Clear ()
+		{
+			Clear (true);
+		}
+
+		public void Clear (bool update)
+		{
+			int [] ids = Ids;
+			selected_cells.Clear ();
+			SignalChange (ids);
+		}
+
+		public void Add (IBrowsableItem item)
+		{
+			if (this.Contains (item))
+				return;
+
+			int index = parent.IndexOf (item);
+			this.Add (index);
+		}
+
+		public int Count {
+			get {
+				return selected_cells.Count;
+			}
+		}
+
+		public bool Contains (IBrowsableItem item)
+		{
+			return selected_cells.ContainsKey (item);
+		}
+
+		public bool Contains (int num)
+		{
+			if (num < 0 || num > parent.Count)
+				return false;
+
+			return this.Contains (parent [num]);
+		}
+		
+		public void Add (int num)
+		{
+			this.Add (num, true);
+		}
+
+		private void Add (int num, bool notify)
+		{
+			if (this.Contains (num))
+			    return;
+			
+			IBrowsableItem item = parent [num];
+			selected_cells [item] = num;
+
+			if (notify)
+				SignalChange (new int [] {num});
+		}
+
+		public void Add (int start, int end)
+		{
+			if (start == -1 || end == -1)
+				return;
+			
+			int current = Math.Min (start, end);
+			int final = Math.Max (start, end);				
+			int count = final - current + 1;
+			int [] ids = new int [count];
+
+			for (int i = 0; i < count; i++) {
+				this.Add (current, false);
+				ids [i] = current;
+				current++;
+			}
+			
+			SignalChange (ids);
+		}
+
+		public int IndexOf (int parent_index)
+		{
+			return System.Array.IndexOf (this.Ids, parent_index);
+		}
+		
+		public int IndexOf (IBrowsableItem item)
+		{
+			if (!this.Contains (item))
+				return -1;
+
+			int parent_index = (int) selected_cells [item];
+			return System.Array.IndexOf (Ids, parent_index);
+		}
+
+		public void Remove (int cell)
+		{
+			IBrowsableItem item = parent [cell];
+			if (item != null)
+				this.Remove (item);
+
+		}
+		
+		public void Remove (IBrowsableItem item)
+		{
+			int parent_index = (int) selected_cells [item];
+			selected_cells.Remove (item);
+			SignalChange (new int [] {parent_index});
+		}
+
+		public event IBrowsableCollectionChangedHandler Changed;
+		public event IBrowsableCollectionItemChangedHandler ItemChanged;
+
+		public delegate void DetailedCollectionChanged (IBrowsableCollection collection, int [] ids);
+		public event DetailedCollectionChanged DetailedChanged;
+		
+		private void ClearCached ()
+		{
+			selection = null;
+			items = null;
+		}
+
+		private void SignalChange (int [] ids) 
+		{
+			ClearCached ();
+			old = this.Items;
+			
+			
+			if (Changed != null)
+				Changed (this);
+			
+			if (DetailedChanged!= null)
+				DetailedChanged (this, ids);
 		}
 	}
 	
-	public bool IdxIsSelected (int num)
-	{
-		return CellIsSelected (num);
-	}
-
 	public int CurrentIdx {
 		get {
-			if (selected_cells.Count == 1 && IdxIsSelected (FocusCell))
+			if (selection.Count == 1 && selection.Contains (FocusCell))
 				return FocusCell;
 			else 
 				return -1;
@@ -227,7 +457,7 @@ public class IconView : Gtk.Layout {
 	// Updating.
 	public void UpdateThumbnail (int thumbnail_num)
 	{
-		FSpot.IBrowsableItem photo = collection.Items [thumbnail_num];
+		FSpot.IBrowsableItem photo = collection [thumbnail_num];
 		string thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 		cache.Remove (thumbnail_path);
 		InvalidateCell (thumbnail_num);
@@ -253,18 +483,23 @@ public class IconView : Gtk.Layout {
 
 		if (x < BORDER_SIZE || x >= BORDER_SIZE + cells_per_row * cell_width)
 			return -1;
-		if (y < BORDER_SIZE || y >= BORDER_SIZE + (collection.Items.Length / cells_per_row + 1) * cell_height)
+		if (y < BORDER_SIZE || y >= BORDER_SIZE + (collection.Count / cells_per_row + 1) * cell_height)
 			return -1;
 
 		int column = (int) ((x - BORDER_SIZE) / cell_width);
 		int row = (int) ((y - BORDER_SIZE) / cell_height);
 		int cell_num = column + row * cells_per_row;
 
-		if (cell_num < collection.Items.Length)
+		if (cell_num < collection.Count)
 			return (int) cell_num;
 		else
 			return -1;
 	}
+	
+	public int TopLeftVisibleCell ()
+	{
+		return CellAtPosition(BORDER_SIZE, (int)Vadjustment.Value + BORDER_SIZE + 8);
+	}
 
 	public void GetCellCenter (int cell_num, out int x, out int y)
 	{
@@ -286,89 +521,17 @@ public class IconView : Gtk.Layout {
 
 
 	// Private utility methods.
-	public void UnselectAllCells ()
-	{
-		if (selected_cells.Count == 0)
-			return;
-
-		if (selected_cells.Count < 100) {
-			foreach (int cell in selected_cells.Keys) {
-				InvalidateCell (cell);
-			}
-		} else {
-			QueueDraw ();
-		}
-		selected_cells.Clear ();		
-
-
-		if (SelectionChanged != null)
-			SelectionChanged (this);
-	}
-
-	public bool CellIsSelected (int cell_num)
-	{
-		return selected_cells.ContainsKey (cell_num);
-	}
-
-	private void SelectCellNoNotify (int cell_num)
-	{
-		if (CellIsSelected (cell_num))
-			return;
-
-		selected_cells.Add (cell_num, cell_num);
-
-		InvalidateCell (cell_num);
-	}
-
-	private void SelectCell (int cell_num)
-	{
-		SelectCellNoNotify (cell_num);
-
-		if (SelectionChanged != null)
-			SelectionChanged (this);
-	}
-
 	public void SelectAllCells ()
 	{
-		SelectCellRange (0, collection.Items.Length - 1);
-	}
-
-	private void SelectCellRange (int start, int end)
-	{
-		if (start == -1 || end == -1)
-			return;
-
-		int current = Math.Min (start, end);
-		int final = Math.Max (start, end);				
-	
-		while (current <= final) {
-			SelectCellNoNotify (current);
-			current++;
-		}
-
-		if (SelectionChanged != null)
-			SelectionChanged (this);
-	}
-
-	private void UnselectCell (int cell_num)
-	{
-		if (! CellIsSelected (cell_num))
-			return;
-
-		selected_cells.Remove (cell_num);
-
-		InvalidateCell (cell_num);
-
-		if (SelectionChanged != null)
-			SelectionChanged (this);
+		selection.Add (0, collection.Count - 1);
 	}
 
 	private void ToggleCell (int cell_num)
 	{
-		if (CellIsSelected (cell_num))
-			UnselectCell (cell_num);
+		if (selection.Contains (cell_num))
+		        selection.Remove (cell_num);
 		else
-			SelectCell (cell_num);
+			selection.Add (cell_num);
 	}
 
 
@@ -383,14 +546,21 @@ public class IconView : Gtk.Layout {
 	}
 	
 	protected virtual void UpdateLayout ()
+	{
+		UpdateLayout (Allocation);
+	}
+
+	protected virtual void UpdateLayout (Gdk.Rectangle Allocation)
 	{
 		int available_width = Allocation.Width - 2 * BORDER_SIZE;
 
-		cell_width = ThumbnailWidth + 2 * CELL_BORDER_WIDTH;
-		cell_height = ThumbnailHeight + 2 * CELL_BORDER_WIDTH;
-
+		cell_width = ThumbnailWidth + 2 * cell_border_width;
+		cell_height = ThumbnailHeight + 2 * cell_border_width;
+		cells_per_row = Math.Max ((int) (available_width / cell_width), 1);
+		cell_width += (available_width - cells_per_row * cell_width) / cells_per_row;
+		
 		if (DisplayTags)
-			cell_height += TAG_ICON_SIZE + TAG_ICON_VSPACING;
+			cell_height += tag_icon_size + tag_icon_vspacing;
 		
 		if (DisplayDates && this.Style != null) {
 			Pango.FontMetrics metrics = this.PangoContext.GetMetrics (this.Style.FontDescription, 
@@ -398,11 +568,9 @@ public class IconView : Gtk.Layout {
 			cell_height += PangoPixels (metrics.Ascent + metrics.Descent);
 		}
 
-		cells_per_row = Math.Max ((int) (available_width / cell_width), 1);
-
 		int num_thumbnails;
 		if (collection != null)
-			num_thumbnails = collection.Items.Length;
+			num_thumbnails = collection.Count;
 		else
 			num_thumbnails = 0;
 
@@ -410,10 +578,53 @@ public class IconView : Gtk.Layout {
 		if (num_thumbnails % cells_per_row != 0)
 			num_rows ++;
 
-		SetSize ((uint) Allocation.Width, (uint) (num_rows * cell_height + 2 * BORDER_SIZE));
+		int height = num_rows * cell_height + 2 * BORDER_SIZE;
 
 		Vadjustment.StepIncrement = cell_height;
-		Vadjustment.Change ();
+		int x = (int)(Hadjustment.Value);
+		int y = (int)(height * scroll_value);
+		SetSize (x, y, (int) Allocation.Width, (int) height);
+	}
+
+	void SetSize (int x, int y, int width, int height)
+	{
+		bool xchange = false; 
+		bool ychange = false; 
+
+		Hadjustment.Upper = System.Math.Max (Allocation.Width, width);
+		Vadjustment.Upper = System.Math.Max (Allocation.Height, height);
+
+		if (scroll) {
+			xchange = (int)(Hadjustment.Value) != x;
+			ychange = (int)(Vadjustment.Value) != y;
+			scroll = false;
+		}
+
+		if (IsRealized) 
+			BinWindow.FreezeUpdates ();
+
+		if (xchange || ychange) {
+			if (IsRealized) 
+				BinWindow.MoveResize (-x, -y, (int)(Hadjustment.Upper), (int)(Vadjustment.Upper));
+			Vadjustment.Value = y;
+			Hadjustment.Value = x;
+		}
+
+		if (scroll)
+			scroll = false;
+
+		if (this.Width != Allocation.Width || this.Height != Allocation.Height)
+			SetSize ((uint)Allocation.Width, (uint)height);
+		
+		if (xchange || ychange) {
+			Vadjustment.ChangeValue ();
+			Hadjustment.ChangeValue ();
+		}
+		
+		if (IsRealized) {
+			BinWindow.ThawUpdates ();
+			BinWindow.ProcessUpdates (true);
+		}
 	}
 
 	static Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
@@ -453,7 +664,7 @@ public class IconView : Gtk.Layout {
 		if (!bounds.Intersect (area, out area))
 			return;
 		
-		FSpot.IBrowsableItem photo = collection.Items [thumbnail_num];
+		FSpot.IBrowsableItem photo = collection [thumbnail_num];
 		string thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 		
 		FSpot.PixbufCache.CacheEntry entry = cache.Lookup (thumbnail_path);
@@ -462,7 +673,7 @@ public class IconView : Gtk.Layout {
 		else
 			entry.Data = thumbnail_num;
 
-		bool selected = CellIsSelected (thumbnail_num);
+		bool selected = selection.Contains (thumbnail_num);
 		StateType cell_state = selected ? (HasFocus ? StateType.Selected :StateType.Active) : StateType.Normal;
 		
 		Style.PaintFlatBox (Style, BinWindow, cell_state, 
@@ -480,7 +691,7 @@ public class IconView : Gtk.Layout {
 		}
 
 		Gdk.Rectangle region = Gdk.Rectangle.Zero;
-		Gdk.Rectangle image_bounds = Expand (bounds, - CELL_BORDER_WIDTH);
+		Gdk.Rectangle image_bounds = Expand (bounds, - cell_border_width);
 		int expansion = ThrobExpansion (thumbnail_num, selected);
 
 		Gdk.Pixbuf thumbnail = null;
@@ -493,22 +704,34 @@ public class IconView : Gtk.Layout {
 					 true, out region.Width, out region.Height);
 			
 			region.X = (int) (bounds.X + (bounds.Width - region.Width) / 2);
-			region.Y = (int) bounds.Y + ThumbnailHeight - region.Height + CELL_BORDER_WIDTH;
+			region.Y = (int) bounds.Y + ThumbnailHeight - region.Height + cell_border_width;
 			
 			if (region.Width != thumbnail.Width && region.Height != thumbnail.Height)
 				cache.Reload (entry, thumbnail_num, thumbnail.Width, thumbnail.Height);
 
 			region = Expand (region, expansion);
 			Pixbuf temp_thumbnail;			
+			region.Width = System.Math.Max (1, region.Width);
+			region.Height = System.Math.Max (1, region.Height); 
 
 			if (region.Width != thumbnail.Width && region.Height != thumbnail.Height) {
 				if (region.Width < thumbnail.Width && region.Height < thumbnail.Height) {
+					/*
 					temp_thumbnail = PixbufUtils.ScaleDown (thumbnail, 
 										region.Width, region.Height);
+					*/
+					temp_thumbnail = thumbnail.ScaleSimple (region.Width, region.Height,
+										InterpType.Bilinear);
+									      
 					
-					if (entry.Reload && expansion == 0)
-						cache.Update (entry, PixbufUtils.ShallowCopy (temp_thumbnail));
-					
+					lock (entry) {
+						if (entry.Reload && expansion == 0) {
+							if (entry.Path != null) {
+								entry.Pixbuf = PixbufUtils.ShallowCopy (temp_thumbnail);
+								entry.Reload = true;
+							}
+						}
+					}
 				} else {
 					temp_thumbnail = thumbnail.ScaleSimple (region.Width, region.Height, 
 										InterpType.Bilinear);
@@ -526,11 +749,12 @@ public class IconView : Gtk.Layout {
 			
 			Gdk.Rectangle draw = Expand (region, 1);
 			
-			Style.PaintShadow (Style, BinWindow, cell_state,
-					   ShadowType.Out, area, this, 
-					   "IconView", 
-					   draw.X, draw.Y, 
-					   draw.Width, draw.Height);			
+			if (!temp_thumbnail.HasAlpha) 
+				Style.PaintShadow (Style, BinWindow, cell_state,
+						   ShadowType.Out, area, this, 
+						   "IconView", 
+						   draw.X, draw.Y, 
+						   draw.Width, draw.Height);			
 			
 			if (region.Intersect (area, out draw)) {
 				temp_thumbnail.RenderToDrawable (BinWindow, Style.WhiteGC,
@@ -568,11 +792,11 @@ public class IconView : Gtk.Layout {
 			
 			layout.GetPixelSize (out layout_bounds.Width, out layout_bounds.Height);
 
-			layout_bounds.Y = bounds.Y + bounds.Height - CELL_BORDER_WIDTH - layout_bounds.Height;
+			layout_bounds.Y = bounds.Y + bounds.Height - cell_border_width - layout_bounds.Height;
 			layout_bounds.X = bounds.X + (bounds.Width - layout_bounds.Width) / 2;
 			
 			if (DisplayTags)
-				layout_bounds.Y -= TAG_ICON_SIZE;
+				layout_bounds.Y -= tag_icon_size;
 
 			if (layout_bounds.Intersect (area, out region)) {
 				Style.PaintLayout (Style, BinWindow, cell_state,
@@ -586,12 +810,15 @@ public class IconView : Gtk.Layout {
 			Tag [] tags = photo.Tags;
 			Gdk.Rectangle tag_bounds;
 
-			tag_bounds.X = bounds.X + (bounds.Width  + TAG_ICON_VSPACING - tags.Length * (TAG_ICON_SIZE + TAG_ICON_VSPACING)) / 2;
-			tag_bounds.Y = bounds.Y + bounds.Height - CELL_BORDER_WIDTH - TAG_ICON_SIZE;
-			tag_bounds.Width = TAG_ICON_SIZE;
-			tag_bounds.Height = TAG_ICON_SIZE;
-
+			tag_bounds.X = bounds.X + (bounds.Width  + tag_icon_vspacing - tags.Length * (tag_icon_size + tag_icon_vspacing)) / 2;
+			tag_bounds.Y = bounds.Y + bounds.Height - cell_border_width - tag_icon_size;
+			tag_bounds.Width = tag_icon_size;
+			tag_bounds.Height = tag_icon_size;
+			
 			foreach (Tag t in tags) {
+				if (t == null)
+					continue;
+
 				Pixbuf icon = null;
 				if (t.Category.Icon == null) {
 					if (t.Icon == null)
@@ -623,7 +850,7 @@ public class IconView : Gtk.Layout {
 					if (scaled_icon != icon)
 						scaled_icon.Dispose ();
 				}
-				tag_bounds.X += tag_bounds.Width + TAG_ICON_VSPACING;
+				tag_bounds.X += tag_bounds.Width + tag_icon_vspacing;
 			}
 		}
 	}
@@ -651,12 +878,12 @@ public class IconView : Gtk.Layout {
 		//Preload (area, false);
 
 		for (i = 0, cell_num = start_cell_num;
-		     i < num_rows && cell_num < collection.Items.Length;
+		     i < num_rows && cell_num < collection.Count;
 		     i ++) {
 			int cell_x = start_cell_x;
 
 			//Console.WriteLine ("Drawing row {0}", start_cell_row + i);
-			for (int j = 0; j < num_cols && cell_num + j < collection.Items.Length; j ++) {
+			for (int j = 0; j < num_cols && cell_num + j < collection.Count; j ++) {
 				DrawCell (cell_num + j, area);
 				cell_x += cell_width;
 			}
@@ -788,8 +1015,8 @@ public class IconView : Gtk.Layout {
 		int len = rows * cols;
 		int scell = start_cell_num;
 		int ecell = scell + len;
-		if (scell > collection.Items.Length - len) {
-		        ecell = collection.Items.Length;
+		if (scell > collection.Count - len) {
+		        ecell = collection.Count;
 			scell = System.Math.Max (0, scell - len);
 		} else
 			ecell = scell + len;
@@ -799,7 +1026,7 @@ public class IconView : Gtk.Layout {
 		{
 			int cell = back ? ecell - i - 1 : scell + mid + i;
 
-			photo = collection.Items [cell];
+			photo = collection [cell];
 			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 			
 			entry = cache.Lookup (thumbnail_path);
@@ -807,7 +1034,7 @@ public class IconView : Gtk.Layout {
 				cache.Request (thumbnail_path, cell, ThumbnailWidth, ThumbnailHeight);
 
 			cell = back ? scell + i : scell + mid - i - 1;
-			photo = collection.Items [cell];
+			photo = collection [cell];
 			thumbnail_path = FSpot.ThumbnailGenerator.ThumbnailPath (photo.DefaultVersionUri);
 			
 			entry = cache.Lookup (thumbnail_path);
@@ -865,17 +1092,19 @@ public class IconView : Gtk.Layout {
 
 		GetCellPosition (cell_num, out x, out y);
 
-		if (y < adjustment.Value)
-			adjustment.Value = y; 
-		else if (y + cell_height > adjustment.Value + adjustment.PageSize)
-			adjustment.Value = y + cell_height - adjustment.PageSize;
+		if (y + cell_height > adjustment.Upper)
+			UpdateLayout ();
+
+		adjustment.Value = y + cell_height / 2 - adjustment.PageSize / 2;
 		
-		adjustment.Change ();
+		adjustment.ChangeValue ();
+
 	}
 
 
 	// Event handlers.
 
+	[GLib.ConnectBefore]
 	private void HandleAdjustmentValueChanged (object sender, EventArgs args)
 	{
 		Scroll ();
@@ -886,8 +1115,8 @@ public class IconView : Gtk.Layout {
 		Gdk.Pixbuf result = entry.ShallowCopyPixbuf ();
 		int order = (int) entry.Data;
 
-		if (order > 0 && order < collection.Items.Length) {
-			System.Uri uri = collection.Items [order].DefaultVersionUri;
+		if (order > 0 && order < collection.Count) {
+			System.Uri uri = collection [order].DefaultVersionUri;
 
 			if (result == null && !System.IO.File.Exists (FSpot.ThumbnailGenerator.ThumbnailPath (uri)))
 				FSpot.ThumbnailGenerator.Default.Request (uri.LocalPath, 0, 256, 256);
@@ -928,14 +1157,18 @@ public class IconView : Gtk.Layout {
 
 	public void InvalidateCell (int order) 
 	{
+		if (dead) return;
 		Rectangle cell_area = CellBounds (order);
 		// FIXME where are we computing the bounds incorrectly
 		cell_area.Width -= 1;
 		cell_area.Height -= 1;
-		Gdk.Rectangle visible = new Gdk.Rectangle ((int)Hadjustment.Value, (int)Vadjustment.Value, Allocation.Width, Allocation.Height);
-		if (cell_area.Intersect (visible, out cell_area)) {
+		Gdk.Rectangle visible = new Gdk.Rectangle ((int)Hadjustment.Value, 
+							   (int)Vadjustment.Value, 
+							   Allocation.Width, 
+							   Allocation.Height);
+
+		if (BinWindow != null && cell_area.Intersect (visible, out cell_area))
 			BinWindow.InvalidateRect (cell_area, false);
-		}
 	}
 			
 	private void HandleScrollAdjustmentsSet (object sender, ScrollAdjustmentsSetArgs args)
@@ -946,8 +1179,10 @@ public class IconView : Gtk.Layout {
 
 	protected override void OnSizeAllocated (Gdk.Rectangle allocation)
 	{
+		scroll_value = (Vadjustment.Value)/ (Vadjustment.Upper);
+		scroll = true;
+		UpdateLayout (allocation);
 		base.OnSizeAllocated (allocation);
-		UpdateLayout ();
 	}
 
 	protected override bool OnExposeEvent (Gdk.EventExpose args)
@@ -960,13 +1195,13 @@ public class IconView : Gtk.Layout {
 
  	private void HandleButtonPressEvent (object obj, ButtonPressEventArgs args)
  	{
-		int cell_num = CellAtPosition ((int) args.Event.X, (int) args.Event.Y);
+		int cell_num = CellAtPosition ((int) args.Event.X, (int) args.Event.Y, false);
 
 		args.RetVal = true;
 
 		if (cell_num < 0) {
 			args.RetVal = false;
-			UnselectAllCells ();
+			selection.Clear ();
 			return;
 		}
 
@@ -985,10 +1220,10 @@ public class IconView : Gtk.Layout {
 			if ((args.Event.State & ModifierType.ControlMask) != 0) {
 				ToggleCell (cell_num);
 			} else if ((args.Event.State & ModifierType.ShiftMask) != 0) {
-				SelectCellRange (FocusCell, cell_num);
-			} else if (!CellIsSelected (cell_num)) {
-				UnselectAllCells ();
-				SelectCell (cell_num);
+				selection.Add (FocusCell, cell_num);
+			} else if (!selection.Contains (cell_num)) {
+				selection.Clear ();
+				selection.Add (cell_num);
 			}
 
 			if (args.Event.Button == 3){
@@ -1032,9 +1267,9 @@ public class IconView : Gtk.Layout {
 			case EventType.ButtonRelease:
 				if ((args.Event.State & ModifierType.ControlMask) == 0 &&
 				    (args.Event.State & ModifierType.ShiftMask  ) == 0 &&
-					(selected_cells.Count>1)) {
-					UnselectAllCells ();
-					SelectCell (cell_num);
+					(selection.Count > 1)) {
+					selection.Clear ();
+					selection.Add (FocusCell);
 				}
 	
 				break;
@@ -1071,7 +1306,7 @@ public class IconView : Gtk.Layout {
 			FocusCell = 0;
 			break;
 		case Gdk.Key.End:
-			FocusCell = collection.Items.Length - 1; 
+			FocusCell = collection.Count - 1; 
 			break;
 		case Gdk.Key.space:
 			ToggleCell (FocusCell);
@@ -1079,7 +1314,7 @@ public class IconView : Gtk.Layout {
 		case Gdk.Key.Return:
 			if (DoubleClicked == null)
 				break;
-			if (FocusCell < 0 || FocusCell > collection.Items.Length - 1)
+			if (FocusCell < 0 || FocusCell > collection.Count - 1)
 				break;
 			DoubleClicked (this, FocusCell);
 			break;
@@ -1088,23 +1323,25 @@ public class IconView : Gtk.Layout {
 			return;		
 		}
 		
-		if (FocusCell < 0 || FocusCell > collection.Items.Length - 1) {
+		if (FocusCell < 0 || FocusCell > collection.Count - 1) {
 			FocusCell = focus_old;
 			args.RetVal = false;
 		}	
 
 		if (shift) {
-			SelectCellRange (focus_old, FocusCell);
+			selection.Add (focus_old, FocusCell);
 		} else if (!control) {
-			UnselectAllCells ();
-			SelectCell (FocusCell);
+			selection.Clear ();
+			selection.Add (FocusCell);
 		} 
 	
 		ScrollTo (FocusCell);
 	}
 
-	private void HandleDestroyEvent (object sender, DestroyEventArgs args)
+	bool dead = false;
+	private void HandleDestroyed (object sender, System.EventArgs args)
 	{
+		cache.OnPixbufLoaded -= HandlePixbufLoaded;
 		CancelThrob ();
 	}
 }
