@@ -13,7 +13,7 @@ using System.Text.RegularExpressions;
 using LibGPhoto2;
 
 public class MainWindow {
-        public static MainWindow Toplevel = null;
+        public static MainWindow Toplevel;
 
 	Db db;
 
@@ -911,7 +911,7 @@ public class MainWindow {
 
 	void HandleOpenCommand (object sender, EventArgs e)
 	{
-		FSpot.SingleView single = new FSpot.SingleView ();
+		new FSpot.SingleView ();
 	}
 
 	void HandleImportCommand (object sender, EventArgs e)
@@ -1124,7 +1124,7 @@ public class MainWindow {
                 new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
 			   "Copyright 2003-2005 Novell Inc.",
-                           null, authors, null, translators, null).Show();
+                           null, authors, new string [0], translators, null).Show();
 	}
 
 	void HandleArrangeByTime (object sender, EventArgs args)
@@ -1147,10 +1147,7 @@ public class MainWindow {
 
 	void HandleCloseCommand (object sender, EventArgs args)
 	{
-		// FIXME
-		// Should use Application.Quit(), but for that to work we need to terminate the threads
-		// first too.
-		Environment.Exit (0);
+		this.Window.Destroy ();
 	}
 	
 	void HandleCreateVersionCommand (object obj, EventArgs args)
@@ -1365,6 +1362,12 @@ public class MainWindow {
 			info_vpaned.Show ();
 	}
 
+	public Gtk.Window Window {
+		get {
+			return main_window;
+		}
+	}
+	
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
 		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
@@ -1629,8 +1632,8 @@ public class MainWindow {
 		}
 		
 		String data = paths.ToString ();
-		primary.SetText (data);
-		clipboard.SetText (data);
+		primary.Text = data;
+		clipboard.Text = data;
 	}
 
 	void HandleSetAsBackgroundCommand (object sender, EventArgs args)
@@ -1702,8 +1705,10 @@ public class MainWindow {
 
 	void HandleQueryItemChanged (FSpot.IBrowsableCollection browsable, int item)
 	{
-		if (info_box.Photo == (browsable [item] as Photo))
+		if (photo_view.Item.Index == item) {
+			UpdateMenus ();
 			info_box.Update ();
+		}
 	}
 	//
 	// Handle Main Menu 
