@@ -30,13 +30,21 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	
 	public static int Compare (Photo photo1, Photo photo2)
 	{
-		int result = CompareImportDate (photo1, photo2);
+		int result = photo1.Id.CompareTo (photo2.Id);
+		
+		if (result == 0)
+			return 0;
+		else 
+			result = CompareImportDate (photo1, photo2);
+
 		if (result == 0)
 			result = CompareCurrentDir (photo1, photo2);
 		
 		if (result == 0)
 			result = CompareName (photo1, photo2);
-
+		
+		if (result == 0)
+			result = photo1.Id.CompareTo (photo2.Id);
 		
 		return result;
 	}
@@ -63,6 +71,9 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 			int result = Photo.CompareCurrentDir (p1, p2);
 			
+			if (result == 0)
+				result = CompareName (p1, p2);
+
 			return result;
 		}
 	}
@@ -230,7 +241,8 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			throw new Exception ("Cannot delete original version");
 
 		string path = GetVersionPath (version_id);
-		File.Delete (path);
+		if (File.Exists (path))
+			File.Delete (path);
 		PhotoStore.DeleteThumbnail (path);
 
 		version_names.Remove (version_id);
@@ -275,11 +287,10 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		int num = 1;
 
 		while (true) {
-			string name;
-			if (num == 1)
-				name = "Modified";
-			else
-				name = String.Format ("Modified ({0})", num);
+			string name = Mono.Posix.Catalog.GetPluralString ("Modified", 
+								 "Modified ({0})", 
+								 num);
+			name = String.Format (name, num);
 
 			if (! VersionNameExists (name))
 				return CreateVersion (name, base_version_id, create_file);
@@ -351,6 +362,18 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 			RemoveTag (tag);
 	}	
 
+	public void RemoveCategory (Tag []taglist)
+	{
+		foreach (Tag tag in taglist) {
+			Category cat = tag as Category;
+
+			if (cat != null)
+				RemoveCategory (cat.Children);
+
+			RemoveTag (tag);
+		}
+	}
+
 	public bool HasTag (Tag tag)
 	{
 		if (tags == null)
@@ -373,7 +396,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 
 		// Note that the original version is never stored in the photo_versions table in the
 		// database.
-		AddVersionUnsafely (OriginalVersionId, "Original");
+		AddVersionUnsafely (OriginalVersionId, Mono.Posix.Catalog.GetString ("Original"));
 	}
 
 	public Photo (uint id, uint unix_time, string path)
@@ -406,34 +429,27 @@ public class PhotoStore : DbStore {
 	//
 	// Generates the thumbnail, returns the Pixbuf, and also stores it as a side effect
 	//
-	static Pixbuf GenerateFromExif (string path, string uri)
-	{
-		try {
-			using (Exif.ExifData ed = new Exif.ExifData (path)){
-				return PixbufUtils.GetThumbnail (ed);
-			}
-		} catch {
-			Console.WriteLine ("Exif died, using regular backend.");
-		}
-		return null;
-	}
 
-	public static Pixbuf GenerateThumbnail (string path, bool use_exif)
+	public static Pixbuf GenerateThumbnail (string path)
 	{
 		string uri = UriList.PathToFileUri (path).ToString ();
 		Pixbuf thumbnail = null;
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
 
-		if (use_exif) {
-			thumbnail = GenerateFromExif (path, uri);
+		if (img is FSpot.IThumbnailContainer) {
+			try {
+				thumbnail = ((FSpot.IThumbnailContainer)img).GetEmbeddedThumbnail ();
+			} catch (Exception e) {
+				System.Console.WriteLine (e.ToString ());
+			}
 		}
 
-		// Save EXIF generated thumbnails in a silightly invalid way so that we know to regnerate them.
+		// Save embedded thumbnails in a silightly invalid way so that we know to regnerate them later.
 		if (thumbnail != null) 
 			PixbufUtils.SaveAtomic (thumbnail, Thumbnail.PathForUri (uri, ThumbnailSize.Large), 
-						"png", null, null);
+						"png", new string [] { null} , new string [] { null});
 		else 
 			thumbnail = FSpot.ThumbnailGenerator.Create (path);
-			
 		
 		return thumbnail;
 	}
@@ -441,7 +457,9 @@ public class PhotoStore : DbStore {
 	public static void DeleteThumbnail (string path)
 	{
 		string uri = UriList.PathToFileUri (path).ToString ();
-		File.Delete (Thumbnail.PathForUri (uri, ThumbnailSize.Large));
+		path = Thumbnail.PathForUri (uri, ThumbnailSize.Large);
+		if (File.Exists (path))
+			File.Delete (path);
 	}
 
 	public static void MoveThumbnail (string old_path, string new_path)
@@ -513,33 +531,20 @@ public class PhotoStore : DbStore {
 
 	public Photo Create (string path, out Pixbuf thumbnail)
 	{
-		DateTime time;
-		try {
-			using (Exif.ExifData ed = new Exif.ExifData (path)) {
-				Exif.ExifContent content = ed.GetContents (Exif.ExifIfd.Exif);
-				Exif.ExifEntry entry = content.GetEntry (Exif.ExifTag.DateTimeOriginal);
-				time = Exif.ExifUtil.DateTimeFromString (entry.Value); 
-				time = time.ToUniversalTime ();
-			}			
-		} catch (System.Exception e) {
-			time = File.GetCreationTimeUtc  (path);
-		} 
-
+		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
+		DateTime time = img.Date ();
 		return Create (time, path, out thumbnail);
 	}
 
 	public Photo Create (DateTime time_in_utc, string path, out Pixbuf thumbnail)
 	{
-		if (! path.ToLower().EndsWith (".jpg") && ! path.ToLower().EndsWith (".jpeg"))
-			throw new Exception ("Only jpeg files supported");
-
-
 		uint unix_time = DbUtils.UnixTimeFromDateTime (time_in_utc);
 
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
-		command.CommandText = String.Format ("INSERT INTO photos (time, directory_path, name, description, default_version_id) " +
+		command.CommandText = String.Format ("INSERT INTO photos (time, " +
+						     "directory_path, name, description, default_version_id) " +
 						     "       VALUES ({0}, '{1}', '{2}', '', {3})                                       ",
 						     unix_time,
 						     SqlString (System.IO.Path.GetDirectoryName (path)),
@@ -552,8 +557,9 @@ public class PhotoStore : DbStore {
 		uint id = (uint) Connection.LastInsertRowId;
 		Photo photo = new Photo (id, unix_time, path);
 		AddToCache (photo);
+		photo.Loaded = true;
 
-		thumbnail = GenerateThumbnail (path, true);
+		thumbnail = GenerateThumbnail (path);
 		return photo;
 	}
 
@@ -808,7 +814,7 @@ public class PhotoStore : DbStore {
 		Photo [] photos = Query (tags);	
 
 		foreach (Photo photo in photos) {
-			photo.RemoveTag (tags);
+			photo.RemoveCategory (tags);
 			Commit (photo);
 		}
 		
@@ -1056,11 +1062,12 @@ public class PhotoStore : DbStore {
 		bool hide = true;
 		if (tags != null) {
 			foreach (Tag t in tags) {
-				if (t.Id == tag_store.Hidden.Id) 
+				if (tag_store.Hidden == null || t.Id == tag_store.Hidden.Id) 
 					hide = false;
 			}
-		}
-		
+		} else
+			hide = tag_store.Hidden != null;
+
 		// The SQL query that we want to construct is:
 		//
 		// SELECT photos.id
@@ -1097,23 +1104,26 @@ public class PhotoStore : DbStore {
 		}
 		
 		if (tags != null && tags.Length > 0) {
-				bool first = true;
-				foreach (Tag t in tags) {
-					if (t.Id == tag_store.Hidden.Id)
-						continue;
-					
-					if (first) {
-						query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
-										     hide || range != null ? " AND " : " WHERE "));
-					}
-					
-					query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
-					
-					first = false;
+			bool first = true;
+			foreach (Tag t in tags) {
+				if (t == null)
+					continue;
+				
+				if (tag_store.Hidden != null && t.Id == tag_store.Hidden.Id)
+					continue;
+				
+				if (first) {
+					query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
+									     hide || range != null ? " AND " : " WHERE "));
 				}
 				
-				if (!first)
-					query_builder.Append (")) ");
+				query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
+				
+					first = false;
+			}
+			
+			if (!first)
+				query_builder.Append (")) ");
 		}
 		
 		query_builder.Append ("ORDER BY photos.time");
