@@ -7,26 +7,16 @@ using System;
 public class PhotoView : EventBox {
 	FSpot.Delay description_delay; 
 
-	public int CurrentPhoto {
-		get {
-			return photo_view.CurrentPhoto;
-		}
-		set {
-			photo_view.CurrentPhoto = value;
-		}
-	}
-
 	public FSpot.PhotoImageView View {
 		get {
 			return photo_view;
 		}
 	}
 
-	private bool CurrentPhotoValid () {
-		if (query == null || query.Photos.Length == 0 || CurrentPhoto >= Query.Photos.Length)
-			return false;
-
-		return true;
+	new public FSpot.BrowsablePointer Item {
+		get {
+			return photo_view.Item;
+		}
 	}
 
 	private PhotoStore photo_store;
@@ -113,9 +103,11 @@ public class PhotoView : EventBox {
 		new SelectionConstraint (Mono.Posix.Catalog.GetString ("Square"), 1.0)
 	};
 
+	private System.Collections.Hashtable constraint_table = new System.Collections.Hashtable ();
+
 	private void HandleSelectionConstraintOptionMenuActivated (object sender, EventArgs args)
 	{
-		selection_constraint_ratio_idx = (int) (sender as GLib.Object).Data [CONSTRAINT_RATIO_IDX_KEY];
+		selection_constraint_ratio_idx = (int) constraint_table [sender];
 		photo_view.SelectionXyRatio = constraints [selection_constraint_ratio_idx].XyRatio;
 	}
 
@@ -127,7 +119,7 @@ public class PhotoView : EventBox {
 		foreach (SelectionConstraint c in constraints) {
 			MenuItem menu_item = new MenuItem (c.Label);
 			menu_item.Show ();
-			menu_item.Data.Add (CONSTRAINT_RATIO_IDX_KEY, i);
+			constraint_table [menu_item] = i;
 			menu_item.Activated += new EventHandler (HandleSelectionConstraintOptionMenuActivated);
 
 			menu.Append (menu_item);
@@ -145,9 +137,9 @@ public class PhotoView : EventBox {
 
 	private void UpdateButtonSensitivity ()
 	{
-		bool valid = CurrentPhotoValid ();
-		bool prev = valid && CurrentPhoto > 0;
-		bool next = valid && CurrentPhoto < query.Photos.Length -1;
+		bool valid = photo_view.Item.IsValid;
+		bool prev = valid && Item.Index > 0;
+		bool next = valid && Item.Index < query.Count - 1;
 
 		display_previous_button.Sensitive = prev;
 		display_next_button.Sensitive = next;
@@ -165,19 +157,19 @@ public class PhotoView : EventBox {
 			if (Query.Photos.Length == 0)
 				count_label.Text = String.Format ("{0} of {1}", 0, 0);
 			else 
-				count_label.Text = String.Format ("{0} of {1}", CurrentPhoto + 1, Query.Photos.Length);
+				count_label.Text = String.Format ("{0} of {1}", Item.Index + 1, Query.Count);
 		}
 	}
 
 	private void UpdateDescriptionEntry ()
 	{
 		description_entry.Changed -= HandleDescriptionChanged;
-		if (Query.Photos.Length > 0 && CurrentPhoto < Query.Photos.Length) {
+		if (Item.IsValid) {
 			if (description_entry.Sensitive == false)
 				description_entry.Sensitive = true;
-
-			if (description_entry.Text != Query.Photos[CurrentPhoto].Description) {
-				description_entry.Text = Query.Photos[CurrentPhoto].Description;
+			
+			if (description_entry.Text != Item.Current.Description) {
+				description_entry.Text = Item.Current.Description;
 			}
 		} else {
 			description_entry.Sensitive = false;
@@ -199,20 +191,6 @@ public class PhotoView : EventBox {
 			UpdateFinished (this);
 	}
 
-
-	// Browsing.
-
-	private void DisplayNext ()
-	{
-		photo_view.Next ();
-	}
-
-	private void DisplayPrevious ()
-	{
-		photo_view.Prev ();
-	}
-
-
 	// Event handlers.
 	private void HandleButtonPressEvent (object sender, ButtonPressEventArgs args)
 	{
@@ -225,12 +203,12 @@ public class PhotoView : EventBox {
 
 	private void HandleDisplayNextButtonClicked (object sender, EventArgs args)
 	{
-		DisplayNext ();
+		View.Item.MoveNext ();
 	}
 
 	private void HandleDisplayPreviousButtonClicked (object sender, EventArgs args)
 	{
-		DisplayPrevious ();
+		View.Item.MovePrevious ();
 	}
 
 
@@ -249,15 +227,26 @@ public class PhotoView : EventBox {
 	private void ProcessImage (bool redeye)
 	{
 		int x, y, width, height;
-		if (! photo_view.GetSelection (out x, out y, out width, out height))
+		if (! photo_view.GetSelection (out x, out y, out width, out height)) {
+			string msg = Mono.Posix.Catalog.GetString ("No selection available");
+			string desc = Mono.Posix.Catalog.GetString ("This tool requires an active selection. Please select a region of the image and try the operation again");
+			
+			HigMessageDialog md = new HigMessageDialog ((Gtk.Window)this.Toplevel, DialogFlags.DestroyWithParent, 
+								    Gtk.MessageType.Error, ButtonsType.Ok, 
+								    msg,
+								    desc);
+
+			md.Run ();
+			md.Destroy ();
 			return;
-		
+		}		
+
 		Pixbuf original_pixbuf = photo_view.CompletePixbuf ();
 		if (original_pixbuf == null) {
 			return;
 		}
 
-		Photo photo = query.Photos [CurrentPhoto];
+		Photo photo = (Photo)Item.Current;
 		Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
 
 		Pixbuf edited;
@@ -289,13 +278,13 @@ public class PhotoView : EventBox {
 				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
 						      95, exif_data);
 				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.Commit (CurrentPhoto);
+				query.Commit (Item.Index);
 			} else {
 				// FIXME we need to invalidate the thumbnail in the cache as well
 				PixbufUtils.SaveJpeg (edited, photo.DefaultVersionPath, 
 						      95, exif_data);
 				FSpot.ThumbnailGenerator.Create (photo.DefaultVersionPath).Dispose ();
-				query.MarkChanged (CurrentPhoto);
+				query.MarkChanged (Item.Index);
 			}
 		} catch (System.Exception e) {
 			string msg = Mono.Posix.Catalog.GetString ("Error editing photo");
@@ -329,7 +318,7 @@ public class PhotoView : EventBox {
 		if (description_delay.IsPending) {
 			description_delay.Stop ();
 
-#if UPDATE_EXIF_DESCRIPTION
+#if true // UPDATE_EXIF_DESCRIPTION
 			Photo photo = query.Photos [description_photo];
 			FSpot.ImageFile img = FSpot.ImageFile.Create (photo.DefaultVersionPath);
 			if (img is FSpot.JpegFile) {
@@ -337,7 +326,8 @@ public class PhotoView : EventBox {
 				jimg.Description = photo.Description;
 				jimg.SaveMetaData (photo.DefaultVersionPath);
 			}
-			Query.Store.Commit (photo);
+			//Query.Store.Commit (photo);
+			Query.Commit (description_photo);
 #else
 			Query.Commit (description_photo);
 #endif
@@ -346,18 +336,18 @@ public class PhotoView : EventBox {
 	}
 
 	private void HandleDescriptionChanged (object sender, EventArgs args) {
-		if (!CurrentPhotoValid ())
+		if (!Item.IsValid)
 			return;
 
-		Query.Photos[CurrentPhoto].Description = description_entry.Text;
+		((Photo)Item.Current).Description = description_entry.Text;
 
 		if (description_delay.IsPending)
-			if (description_photo == CurrentPhoto)
+			if (description_photo == Item.Index)
 				description_delay.Stop ();
 			else
 				CommitPendingChanges ();
 
-		description_photo = CurrentPhoto;
+		description_photo = Item.Index;
 		description_delay.Start ();
 	}
 
@@ -377,11 +367,9 @@ public class PhotoView : EventBox {
 	private void HandlePhotoChanged (FSpot.PhotoImageView view)
 	{
 		CommitPendingChanges ();
-		bool valid = CurrentPhotoValid ();
-
 		Update ();
 
-		tag_view.Current = valid ? query.Photos [CurrentPhoto] : null;
+		tag_view.Current = (Photo)(Item.Current);
 
 		if (this.PhotoChanged != null)
 			PhotoChanged (this);
@@ -392,6 +380,8 @@ public class PhotoView : EventBox {
 		CommitPendingChanges ();
 	}
 
+	Gtk.Tooltips tips = new Gtk.Tooltips ();
+
 	public PhotoView (FSpot.PhotoQuery query, PhotoStore photo_store)
 		: base ()
 	{
@@ -485,9 +475,15 @@ public class PhotoView : EventBox {
 		display_next_button.Clicked += new EventHandler (HandleDisplayNextButtonClicked);
 		toolbar_hbox.PackStart (display_next_button, false, true, 0);
 
+		tips.Enable ();
+
+
 		UpdateButtonSensitivity ();
 
 		vbox.ShowAll ();
+		tips.SetTip (crop_button, Mono.Posix.Catalog.GetString ("Crop image to selected area"), "so is this one");
+		tips.SetTip (redeye_button, Mono.Posix.Catalog.GetString ("Remove redeye from selected area"), "this is one too");
+		tips.SetTip (color_button, Mono.Posix.Catalog.GetString ("Adjust the image colors"), "This is a tip");
 	}
 
 }
